@@ -1,19 +1,12 @@
 package SVK::Command;
 use strict;
-use base qw(App::CLI App::CLI::Command);
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 use Getopt::Long qw(:config no_ignore_case bundling);
 
 use SVK::Util qw( get_prompt abs2rel abs_path is_uri catdir bsd_glob from_native
-		  find_svm_source $SEP IS_WIN32 catdepot traverse_history);
+		  find_svm_source $SEP IS_WIN32 HAS_SVN_MIRROR catdepot traverse_history);
 use SVK::I18N;
 use Encode;
-use constant subcommands => '*';
-
-use Class::Autouse
-    qw( Path::Class SVK::Path SVK::Path::Checkout SVK::Notify
-	SVK::Editor::Status SVK::Editor::Diff
-	Pod::Simple::Text SVK::Merge );
 
 =head1 NAME
 
@@ -40,8 +33,7 @@ information is displayed instead.
 
 =cut
 
-use constant alias =>
-            qw( ann		annotate
+my %alias = qw( ann		annotate
                 blame		annotate
                 praise		annotate
 		co		checkout
@@ -80,17 +72,67 @@ use constant alias =>
 		ver		version
 	    );
 
-use constant global_options => ( 'h|help|?'   => 'help',
-				 'encoding=s' => 'encoding',
-				 'ignore=s@'  => 'ignore',
-			       );
-
-my %alias = alias;
 my %cmd2alias = map { $_ => [] } values %alias;
 while( my($alias, $cmd) = each %alias ) {
     push @{$cmd2alias{$cmd}}, $alias;
 }
 
+
+=head3 new ($xd)
+
+Base constructor for all commands.
+
+=cut
+
+sub new {
+    my ($class, $xd) = @_;
+    my $self = bless { xd => $xd }, $class;
+    return $self;
+}
+
+=head3 get_cmd ($cmd, $xd)
+
+Load the command subclass specified in C<$cmd>, and return a new
+instance of it, populated with C<$xd>.  Command aliases are handled here.
+
+To construct a command object from another command object, use the
+C<command> instance method instead.
+
+=cut
+
+sub get_cmd {
+    my ($pkg, $cmd, $xd) = @_;
+    die "Command not recognized, try $0 help.\n"
+	unless $cmd =~ m/^[?a-z]+$/;
+    $pkg = join('::', 'SVK::Command', _cmd_map ($cmd));
+    my $file = "$pkg.pm";
+    $file =~ s!::!/!g;
+
+    unless (eval {require $file; 1} and $pkg->can('run')) {
+	warn $@ if $@ and exists $INC{$file};
+	die "Command not recognized, try $0 help.\n";
+    }
+    $pkg->new ($xd);
+}
+
+sub _cmd_map {
+    my ($cmd) = @_;
+    $cmd = $alias{$cmd} if exists $alias{$cmd};
+    $cmd =~ s/^(.)/\U$1/;
+    return $cmd;
+}
+
+# rebless to subcommand class if it exists
+sub _subcommand {
+    my ($self) = @_;
+    no strict 'refs';
+    for (grep {$self->{$_}} values %{{$self->options}}) {
+	if (exists ${ref($self).'::'}{$_.'::'}) {
+	    return bless ($self, (ref($self)."::$_"));
+	}
+    }
+}
+
 =head3 invoke ($xd, $cmd, $output_fh, @args)
 
 Takes a L<SVK::XD> object, the command name, the output scalar reference,
@@ -106,87 +148,57 @@ sub invoke {
     my ($pkg, $xd, $cmd, $output, @args) = @_;
     my ($help, $ofh, $ret);
     my $pool = SVN::Pool->new_default;
-
-    local *ARGV = [$cmd, @args];
     $ofh = select $output if $output;
-    $ret = eval {$pkg->dispatch ($xd ? (xd => $xd) : (),
-				 output => $output) };
-
-    $ofh = select STDERR unless $output;
-    print $ret if $ret && $ret !~ /^\d+$/;
-    unless (ref($@)) {
-	if ($SVN::Core::VERSION gt '1.2.2') {
-	    print $@ if $@;
-	}
-	else {
-	    # if an error handler terminates editor call, there will be stack trace
-	    print $@ if $@ && $@ !~ m/\n.+\n.+\n/
-	}
-    }
-    $ret = 1 if ($ret ? $ret !~ /^\d+$/ : $@);
-
-    undef $pool;
-    select $ofh if $ofh;
-    return ($ret || 0);
-}
-
-sub run_command {
-    my ($self, @args) = @_;
-    my $ret;
-
     local $SVN::Error::handler = sub {
 	my $error = $_[0];
 	my $error_message = $error->expanded_message();
 	$error->clear();
-	if ($self->handle_error ($error)) {
-	    die \'error handled';
-        }
-
-        if ($ENV{SVKSVNBACKTRACE}) {
-            require Carp;
-            Carp::confess($error_message);
-        } else {
-            die $error_message."\n";
-        }
+	$cmd->handle_error ($error) if defined $cmd;
+	die $error_message."\n";
     };
 
-    # XXX: this eval is too nasty
+    local $@;
     eval {
+	$cmd = get_cmd ($pkg, $cmd, $xd);
+	$cmd->{svnconfig} = $xd->{svnconfig} if $xd;
+	$cmd->getopt (\@args, 'h|help|?' => \$help);
+	$cmd->_subcommand;
+
 	# Fake shell globbing on Win32 if we are called from main
 	if (IS_WIN32 and caller(1) eq 'main') {
 	    @args = map {
 		/[?*{}\[\]]/
 		    ? bsd_glob($_, File::Glob::GLOB_NOCHECK())
-			: $_
-		    } @args;
+		    : $_
+	    } @args;
 	}
-	# XXX: xd needs to know encoding and ignore too
-	$self->{xd}{encoding} = $self->{encoding}
-	    if $self->{xd};
-	$self->{xd}{ignore} = $self->{ignore}
-	    if $self->{ignore};
-	if ($self->{help} || !(@args = $self->parse_arg(@args))) {
-	    select STDERR unless $self->{output};
-	    $self->usage;
+
+	if ($help || !(@args = $cmd->parse_arg(@args))) {
+	    select STDERR unless $output;
+	    $cmd->usage;
 	}
 	else {
-	    $self->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
-	    eval { $self->lock (@args);
-		   $self->{xd}->store if $self->{xd} && !$self->{hold_giant};
-		   $ret = $self->run (@args) };
-	    $self->{xd}->unlock if $self->{xd};
+	    $cmd->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
+	    eval { $cmd->lock (@args);
+		   $xd->giant_unlock if $xd && !$cmd->{hold_giant};
+		   $ret = $cmd->run (@args) };
+	    $xd->unlock if $xd;
 	    die $@ if $@;
 	}
-
     };
-    # in case parse_arg dies, unlock giant
-    $self->{xd}->giant_unlock if $self->{xd} && ref ($self) && !$self->{hold_giant};
-    die $@ if $@;
-    return $ret;
-}
 
-sub error_cmd {
-    loc ("Command not recognized, try %1 help.\n", $0);
+    # in case parse_arg dies
+    $xd->giant_unlock if $xd && ref ($cmd) && !$cmd->{hold_giant};
+
+    $ofh = select STDERR unless $output;
+    print $ret if $ret && $ret !~ /^\d+$/;
+    # if an error handler terminates editor call, there will be stack trace
+    print $@ if $@ && $@ !~ m/\n.+\n.+\n/;
+    $ret = 1 if ($ret ? $ret !~ /^\d+$/ : $@);
+    undef $cmd; undef $pool; # this needs to happen before finish select
+    select $ofh if $ofh;
+
+    return ($ret || 0);
 }
 
 =head3 getopt ($argv, %opt)
@@ -211,16 +223,6 @@ sub getopt {
 	if defined $recursive;
 }
 
-sub command_options {
-    my $self = shift;
-    $self->{recursive} = $self->opt_recursive;
-    my %opt;
-    $opt{$self->{recursive} ? 'N|non-recursive' : 'R|recursive'} =
-	sub { $self->{recursive} = !$self->{recursive} }
-	    if defined $self->{recursive};
-    ($self->options, %opt);
-}
-
 =head2 Instance Methods
 
 C<SVK::Command-E<gt>invoke> loads the corresponding class
@@ -243,6 +245,13 @@ value will be stored in C<recursive>.
 
 =cut
 
+sub options { () }
+
+sub _opt_map {
+    my ($self, %opt) = @_;
+    return map {$_ => \$self->{$opt{$_}}} sort keys %opt;
+}
+
 =head3 parse_arg (@args)
 
 This method is called with the remaining arguments after parsing named
@@ -282,7 +291,7 @@ sub run {
 =head2 Utility Methods
 
 Except for C<arg_depotname>, all C<arg_*> methods below returns a
-L<SVK::Path> object, which consists of a hash with the following keys:
+L<SVK::Target> object, which consists of a hash with the following keys:
 
 =over
 
@@ -313,22 +322,41 @@ Argument is a number of checkout paths.
 =cut
 
 sub arg_condensed {
-    my $self = shift;
-    my @args = map { $self->arg_copath($_) } @_;
+    my ($self, @arg) = @_;
+    return if $#arg < 0;
+
+    s{[/\Q$SEP\E]$}{}o for @arg; # XXX band-aid
+
+    my ($report, $copath, @targets )= $self->{xd}->condense (@arg);
+
     if ($self->{recursive}) {
 	# remove redundant targets when doing recurisve
 	# if have '' in targets then it means everything
-	my @newtarget = @args;
-	for my $anchor (sort {length $a->copath <=> length $b->copath} @args) {
-	    local $SIG{__WARN__} = sub {};# path::class bug on /foo/bar vs /foo
-	    @newtarget = grep {
-		$anchor->copath eq $_->copath ||
-		!Path::Class::dir($anchor->copath)->subsumes($_->copath) } @newtarget;
+	my @newtarget = @targets;
+	for my $anchor (sort {length $a <=> length $b} @targets) {
+	    @newtarget = grep { length $anchor ? $_ eq $anchor || index ($_, "$anchor/") != 0
+				               : 0} @newtarget;
 	}
-	@args = @newtarget;
+	@targets = @newtarget;
     }
 
-    return $self->{xd}->target_condensed(@args);
+    my ($repospath, $path, undef, $cinfo, $repos) = $self->{xd}->find_repos_from_co ($copath, 1);
+    my $target = SVK::Target->new
+	( repos => $repos,
+	  repospath => $repospath,
+	  depotpath => $cinfo->{depotpath},
+	  copath => $copath,
+	  path => $path,
+	  report => $report,
+	  targets => @targets ? \@targets : undef );
+    my $root = $target->root ($self->{xd});
+    until ($root->check_path ($target->{path}) == $SVN::Node::dir) {
+	my $targets = delete $target->{targets};
+	$target->anchorify;
+	$target->{targets} = [map {"$target->{targets}[0]/$_"} @$targets]
+	    if $targets;
+    }
+    return $target;
 }
 
 =head3 arg_uri_maybe ($arg, $no_new_mirror)
@@ -343,22 +371,28 @@ sub arg_uri_maybe {
     my ($self, $arg, $no_new_mirror) = @_;
 
     is_uri($arg) or return $self->arg_depotpath($arg);
+    HAS_SVN_MIRROR or die loc("cannot load SVN::Mirror");
 
     $arg =~ s{/?$}{/}; # add a trailing slash at the end
 
     require URI;
     my $uri = URI->new($arg)->canonical or die loc("%1 is not a valid URI.\n", $arg);
     my $map = $self->{xd}{depotmap};
-    foreach my $depotname (sort keys %$map) {
-        my $depot = eval { $self->{xd}->find_depot($depotname) } or next;
-	foreach my $path ($depot->mirror->entries) {
-	    my $m = $depot->mirror->get($path);
-            my $rel_uri = $uri->rel(URI->new($m->url."/")->canonical) or next;
+    foreach my $depot (sort keys %$map) {
+        my $repos = eval { ($self->{xd}->find_repos ("/$depot/", 1))[2] } or next;
+	foreach my $path ( SVN::Mirror::list_mirror ($repos) ) {
+	    my $m = eval {SVN::Mirror->new (
+                repos => $repos,
+                get_source => 1,
+                target_path => $path,
+            ) } or next;
+
+            my $rel_uri = $uri->rel(URI->new("$m->{source}/")->canonical) or next;
             next if $rel_uri->eq($uri);
             next if $rel_uri =~ /^\.\./;
 
-            my $depotpath = catdepot($depot->depotname, $path, $rel_uri);
-            $depotpath = "/$depotpath" if !length($depot->depotname);
+            my $depotpath = catdepot($depot, $path, $rel_uri);
+            $depotpath = "/$depotpath" if !length($depot);
             return $self->arg_depotpath($depotpath);
 	}
     }
@@ -428,9 +462,10 @@ usually good enough.
     # If we're mirroring via svn::mirror, not mirroring the whole history
     # is an option
     my ($m, $answer);
-    $m = $target->is_mirrored;
-    # If the user is mirroring from svn
-    if ($m) {
+    ($m,undef) = SVN::Mirror::is_mirrored ($target->{'repos'}, 
+                                           $target->{'path'}) if (HAS_SVN_MIRROR);
+    # If the user is mirroring from svn                                       
+    if (UNIVERSAL::isa($m,'SVN::Mirror::Ra'))  {                                
         print loc("
 svk needs to mirror the remote repository so you can work locally.
 If you're mirroring a single branch, it's safe to use any of the options
@@ -469,7 +504,7 @@ break history-sensitive merging within the mirrored path.
         }
     )->run ($target);
 
-    my $depotpath = length ($rel_uri) ? $target->depotpath."/$rel_uri" : $target->depotpath;
+    my $depotpath = length ($rel_uri) ? "$target->{depotpath}/$rel_uri" : $target->{depotpath};
     return $self->arg_depotpath($depotpath);
 }
 
@@ -483,29 +518,21 @@ handles it via C<arg_uri_maybe>.
 sub arg_co_maybe {
     my ($self, $arg, $no_new_mirror) = @_;
 
-    $arg = $self->arg_uri_maybe($arg, $no_new_mirror)->depotpath
+    $arg = $self->arg_uri_maybe($arg, $no_new_mirror)->{depotpath}
 	if is_uri($arg);
 
     my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
     my ($repospath, $path, $copath, $cinfo, $repos) =
 	$self->{xd}->find_repos_from_co_maybe ($arg, 1);
     from_native ($path, 'path', $self->{encoding});
-    my ($view, $revision, $subpath);
-    if (($view, $revision, $subpath) = $path =~ m{^/\^([\w/\-_]+)(?:\@(\d+)(.*))?$}) {
-	$revision ||= $repos->fs->youngest_rev;
-	($path, $view) = SVK::Command->create_view ($repos, $view, $revision, $subpath);
-    }
-
-    $rev ||= $cinfo->{revision} if defined $copath;
-    return $self->{xd}->create_path_object
+    return SVK::Target->new
 	( repos => $repos,
 	  repospath => $repospath,
 	  depotpath => $cinfo->{depotpath} || $arg,
+	  copath => $copath,
+	  report => $copath ? File::Spec->canonpath ($arg) : $arg,
 	  path => $path,
-	  view => $view,
 	  revision => $rev,
-	  copath_anchor => $copath,
-	  report => File::Spec->canonpath($arg),
 	);
 }
 
@@ -518,30 +545,16 @@ Argument is a checkout path.
 sub arg_copath {
     my ($self, $arg) = @_;
     my ($repospath, $path, $copath, $cinfo, $repos) = $self->{xd}->find_repos_from_co ($arg, 1);
-    my ($root);
-    my ($view, $rev, $subpath);
-
-    if ($copath =~ m/([\x00-\x19\x7f])/) { # XXX: why isn't \c[ working?
-	die loc("Invalid control character '%1' in path '%2'\n",
-		sprintf("0x%02X", ord($1)), $arg);
-    }
-
-    if (($view, $rev, $subpath) = $path =~ m{^/\^([\w/\-_]+)(?:\@(\d+)(.*))?$}) {
-	($path, $view) = $self->create_view ($repos, $view, $rev, $subpath);
-    }
-
     from_native ($path, 'path', $self->{encoding});
-    return SVK::Path::Checkout->real_new
-	({ report => File::Spec->canonpath ($arg),
-	   copath_anchor => $copath,
-	   xd => $self->{xd},
-	   source => $self->{xd}->create_path_object
-	   ( repos => $repos,
-	     repospath => $repospath,
-	     path => $path,
-	     view => $view,
-	     revision => $cinfo->{revision}, # make this sane!
-	     depotpath => $cinfo->{depotpath} ) });
+    return SVK::Target->new
+	( repos => $repos,
+	  repospath => $repospath,
+	  report => File::Spec->canonpath ($arg),
+	  copath => $copath,
+	  path => $path,
+	  cinfo => $cinfo,
+	  depotpath => $cinfo->{depotpath},
+	);
 }
 
 =head3 arg_depotpath ($arg)
@@ -550,82 +563,19 @@ Argument is a depotpath, including the slashes and depot name.
 
 =cut
 
-sub _resolve_anchor {
-    my ($repos, $base, $anchor) = @_;
-    # XXX
-    $anchor = Path::Class::Dir->new_foreign('Unix', $anchor);
-    $anchor =~ s/^\&\:// or return $anchor;
-    $anchor = Path::Class::Dir->new_foreign('Unix', $anchor);
-    my ($uuid, $path) = find_svm_source($repos, "$base");
-    return $anchor->relative($path)->absolute($base);
-}
-
-sub create_view {
-    my ($self, $repos, $view, $rev, $subpath) = @_;
-    my $fs = $repos->fs;
-    my $viewspec = Path::Class::File->new_foreign('Unix', '/', "$view");
-    my ($viewbase, $viewname) = ($viewspec->parent, $viewspec->basename);
-    $rev = $fs->youngest_rev unless defined $rev;
-    require SVK::View;
-    my $viewobj = SVK::View->new
-	({ name => $viewname, base => $viewbase,
-	   revision => $rev, pool => SVN::Pool->new });
-    $viewobj->pool(SVN::Pool->new);
-    my $root = $fs->revision_root($rev);
-    my $content = $root->node_prop("$viewbase", "svk:view:$viewname");
-    die loc("Unable to create view '%1' from on %2 for revision %3.\n",
-	    $viewname, $viewbase, $rev)
-	unless defined $content;
-    my ($anchor, @content) = grep { $_ && !m/^#/ } $content =~ m/^.*$/mg;
-    $anchor = _resolve_anchor($repos, $viewbase, $anchor);
-    die loc("Unable to create view '%1' from on %2 for revision %3.\n",
-	    $viewname, $viewbase, $rev)
-	unless $root->check_path("$anchor");
-    $viewobj->anchor($anchor);
-
-    $root->dir_entries("$anchor"); # XXX: for some reasons fsfs needs refresh
-
-    for (@content) {
-	my ($del, $path, $target) = m/\s*(-)?(\S+)\s*(\S+)?\s*$/ or die "can't parse $_";
-	my $abspath = Path::Class::Dir->new_foreign('Unix', $path)
-	    ->absolute($anchor);
-	if (defined $target) {
-	    $target = Path::Class::Dir->new_foreign('Unix', $target);
-	    $target = $target->absolute($anchor)
-		unless $target->is_absolute;
-	}
-	if ($del) {
-	    warn "path not required" if defined $target;
-	    $viewobj->add_map($abspath, undef);
-	}
-	else {
-	    die "path required" unless defined $target;
-	    $viewobj->add_map($abspath, $target);
-	}
-    }
-
-    $subpath = '' unless defined $subpath;
-    return (length $subpath ? $anchor eq '/' ? $subpath : $anchor.$subpath
-	    : $anchor->stringify, $viewobj);
-}
-
 sub arg_depotpath {
     my ($self, $arg) = @_;
-    my $root;
-    my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
-    my ($depot, $path) = $self->{xd}->find_depotpath($arg);
-    my $view;
-    from_native ($path, 'path', $self->{encoding});
-    if (($view) = $path =~ m{^/\^([\w\-_/]+)$}) {
-	($path, $view) = $self->create_view($depot->repos, $view, $rev);
-    }
 
-    return $self->{xd}->create_path_object
-	( depot => $depot,
+    my $rev = $arg =~ s/\@(\d+)$// ? $1 : undef;
+    my ($repospath, $path, $repos) = $self->{xd}->find_repos ($arg, 1);
+    from_native ($path, 'path', $self->{encoding});
+    return SVK::Target->new
+	( repos => $repos,
+	  repospath => $repospath,
 	  path => $path,
 	  report => $arg,
 	  revision => $rev,
-	  view => $view,
+	  depotpath => $arg,
 	);
 }
 
@@ -642,7 +592,9 @@ sub arg_depotroot {
     local $@;
     $arg = eval { $self->arg_co_maybe ($arg || '')->new (path => '/') }
            || $self->arg_depotpath ("//");
-    return $arg->as_depotpath->refresh_revision;
+    $arg->as_depotpath;
+
+    return $arg;
 }
 
 =head3 arg_depotname ($arg)
@@ -670,20 +622,6 @@ sub arg_path {
     return abs_path ($arg);
 }
 
-=head3 apply_revision($target)
-
-Apply the given revision from command line to C<$target>.
-
-=cut
-
-sub apply_revision {
-    my ($self, $target) = @_;
-    $target = $target->source if $target->isa('SVK::Path::Checkout');
-    return $target unless defined $self->{rev};
-
-    return $target->seek_to( $self->resolve_revision($target, $self->{rev}) );
-}
-
 =head3 parse_revlist ()
 
 Parse -c or -r to a list of [from, to] pairs.
@@ -691,17 +629,32 @@ Parse -c or -r to a list of [from, to] pairs.
 =cut
 
 sub parse_revlist {
-    my ($self,$target) = @_;
+    my $self = shift;
     die loc("Revision required.\n") unless $self->{revspec} or $self->{chgspec};
     die loc("Can't assign --revision and --change at the same time.\n")
 	if $self->{revspec} and $self->{chgspec};
     my ($fromrev, $torev);
-
-    my @revlist = $self->resolve_chgspec($target);
-    return @revlist if(@revlist);
+    if ($self->{chgspec}) {
+	my @revlist;
+	for (split (',', $self->{chgspec})) {
+	    my $reverse;
+	    if (($fromrev, $torev) = m/^(\d+)-(\d+)$/) {
+		--$fromrev;
+	    }
+	    elsif (($reverse, $torev) = m/^(-?)(\d+)$/) {
+		$fromrev = $torev - 1;
+		($fromrev, $torev) = ($torev, $fromrev) if $reverse;
+	    }
+	    else {
+		die loc("Change spec %1 not recognized.\n", $_);
+	    }
+	    push @revlist , [$fromrev, $torev];
+	}
+	return @revlist;
+    }
 
     # revspec
-    if (($fromrev, $torev) = $self->resolve_revspec($target)) {
+    if (($fromrev, $torev) = $self->{revspec} =~ m/^(\d+):(\d+)$/) {
 	return ([$fromrev, $torev]);
     }
     else {
@@ -721,31 +674,11 @@ XXX Undocumented
 sub lock_target {
     my $self = shift;
     for my $target (@_) {
-	$self->{xd}->lock ($target->copath_anchor)
-	    if $target->isa('SVK::Path::Checkout');
+	$self->{xd}->lock ($target->{copath})
+	    if $target->{copath};
     }
 }
 
-=head3 lock_coroot ($target)
-
-XXX Undocumented
-
-=cut
-
-sub lock_coroot {
-    my $self = shift;
-    my @tgt = map { $_->copath($_->{copath_target}) }
-	grep { $_->isa('SVK::Path::Checkout') } @_;
-    return unless @tgt;
-    my %roots;
-    for (@tgt) {
-	my (undef, $coroot) = $self->{xd}{checkout}->get($_);
-	$roots{$coroot}++;
-    }
-    $self->{xd}->lock($_)
-	for keys %roots;
-}
-
 =head3 brief_usage ($file)
 
 Display an one-line brief usage of the command object.  Optionally, a file
@@ -755,7 +688,9 @@ could be given to extract the usage from the POD.
 
 sub brief_usage {
     my ($self, $file) = @_;
-    open my ($podfh), '<', ($file || $self->filename) or return;
+    my $fname = ref($self);
+    $fname =~ s|::|/|g;
+    open my ($podfh), '<', ($file || $INC{"$fname.pm"}) or return;
     local $/=undef;
     my $buf = <$podfh>;
     if($buf =~ /^=head1\s+NAME\s*SVK::Command::(\w+ - .+)$/m) {
@@ -764,20 +699,6 @@ sub brief_usage {
     close $podfh;
 }
 
-=head3 filename
-
-Return the filename for the command module.
-
-=cut
-
-sub filename {
-    my $self = shift;
-    my $fname = ref($self);
-    $fname =~ s{::[a-z]+}{}; # subcommand
-    $fname =~ s{::}{/}g;
-    $INC{"$fname.pm"}
-}
-
 =head3 usage ($want_detail)
 
 Display usage.  If C<$want_detail> is true, the C<DESCRIPTION>
@@ -787,12 +708,16 @@ section is displayed as well.
 
 sub usage {
     my ($self, $want_detail) = @_;
-    my $fname = $self->filename;
-    my($cmd) = $fname =~ m{\W(\w+)\.pm$};
+    # XXX: the order from selected is not preserved.
+    my $fname = ref($self);
+    $fname =~ s|::|/|g;
+
+    my($cmd) = $fname =~ m{\W(\w+)$};
+
     my $parser = Pod::Simple::Text->new;
     my $buf;
     $parser->output_string(\$buf);
-    $parser->parse_file($fname);
+    $parser->parse_file($INC{"$fname.pm"});
 
     $buf =~ s/SVK::Command::(\w+)/\l$1/g;
     $buf =~ s/^AUTHORS.*//sm;
@@ -901,7 +826,7 @@ sub command {
 
     my $cmd = (
         $is_rebless ? bless($self, "SVK::Command::$command")
-                    : "SVK::Command::$command"->new (xd => $self->{xd})
+                    : "SVK::Command::$command"->new ($self->{xd})
     );
     $cmd->{$_} = $args->{$_} for sort keys %$args;
 
@@ -924,14 +849,14 @@ sub rebless {
 sub find_checkout_anchor {
     my ($self, $target, $track_merge, $track_sync) = @_;
 
-    my $entry = $self->{xd}{checkout}->get ($target->copath_anchor);
+    my $entry = $self->{xd}{checkout}->get ($target->{copath});
     my $anchor_target = $self->arg_depotpath ($entry->{depotpath});
 
     return ($anchor_target, undef) unless $track_merge;
 
     my @rel_path = split(
         '/',
-        abs2rel ($target->path_anchor, $anchor_target->path_anchor, undef, '/')
+        abs2rel ($target->{path}, $anchor_target->{path}, undef, '/')
     );
 
     my $copied_from;
@@ -994,63 +919,20 @@ svk use the default.
     return $path;
 }
 
-## Resolve the correct revision numbers given by "-c"
-sub resolve_chgspec {
-    my ($self,$target) = @_;
-    my @revlist;
-    my ($fromrev,$torev);
-    if(my $chgspec = $self->{chgspec}) {
-	for (split (',', $self->{chgspec})) {
-	    my $reverse;
-	    if (($fromrev, $torev) = m/^(\d+)-(\d+)$/) {
-		--$fromrev;
-	    }
-	    elsif (($reverse, $torev) = m/^(-?)(\d+)$/) {
-		$fromrev = $torev - 1;
-		($fromrev, $torev) = ($torev, $fromrev) if $reverse;
-	    }
-	    else {
-		eval { $torev = $self->resolve_revision($target,$_); };
-		die loc("Change spec %1 not recognized.\n", $_) if($@);
-		$fromrev = $torev - 1;
-	    }
-	    push @revlist , [$fromrev, $torev];
-	}
-    }
-    return @revlist;
-}
-
-# Looks in revspec or change (not in chgspec, which is more complicated)
-
 sub resolve_revspec {
     my ($self,$target) = @_;
-    my $fs = $target->repos->fs;
+    my $fs = $target->{repos}->fs;
     my $yrev = $fs->youngest_rev;
     my ($r1,$r2);
-    die loc("Can't assign --revision and --change at the same time.\n")
-      if defined $self->{revspec} and defined $self->{change};
     if (my $revspec = $self->{revspec}) {
         if ($#{$revspec} > 1) {
-            die loc ("Invalid -r.\n");
+            die loc ("Invliad -r.\n");
         } else {
             $revspec = [map {split /:/} @$revspec];
             ($r1, $r2) = map {
                 $self->resolve_revision($target,$_);
             } @$revspec;
         }
-    } elsif (defined(my $change = $self->{change})) {
-        my $flip;
-        $flip = 1 if $change =~ s/^-//;
-
-        my $r = $self->resolve_revision($target, $change);
-
-        if ($r == 0) {
-            die loc("There is no change 0.\n");
-        } elsif ($flip) {
-            ($r1, $r2) = ($r, $r-1);
-        } else {
-            ($r1, $r2) = ($r-1, $r);
-        }
     }
     return($r1,$r2);
 }
@@ -1058,24 +940,22 @@ sub resolve_revspec {
 sub resolve_revision {
     my ($self,$target,$revstr) = @_;
     return unless defined $revstr;
-    my $fs = $target->repos->fs;
+    my $fs = $target->{repos}->fs;
     my $yrev = $fs->youngest_rev;
     my $rev;
-    if($revstr =~ /^HEAD$/i) {
+    if($revstr =~ /^HEAD$/) {
         $rev = $self->find_head_rev($target);
-    } elsif ($revstr =~ /^BASE$/i) {
+    } elsif ($revstr =~ /^BASE$/) {
         $rev = $self->find_base_rev($target);
     } elsif ($revstr =~ /\{(\d\d\d\d-\d\d-\d\d)\}/) { 
         my $date = $1; $date =~ s/-//g;
         $rev = $self->find_date_rev($target,$date);
-    } elsif ((my ($rrev) = $revstr =~ m'^(\d+)@$')) {
-	if (my $m = $target->is_mirrored) {
+    } elsif (HAS_SVN_MIRROR && (my ($rrev) = $revstr =~ m'^(\d+)@$')) {
+	if (my ($m) = SVN::Mirror::is_mirrored ($target->{repos}, $target->{path})) {
 	    $rev = $m->find_local_rev ($rrev);
 	}
 	die loc ("Can't find local revision for %1 on %2.\n", $rrev, $target->path)
 	    unless defined $rev;
-    } elsif ($revstr =~ /^-\d+$/) {
-        $rev = $self->find_head_rev($target) + $revstr;
     } elsif ($revstr =~ /\D/) {
         die loc("%1 is not a number.\n",$revstr)
     } else {
@@ -1087,10 +967,10 @@ sub resolve_revision {
 sub find_date_rev {
     my ($self,$target,$date) = @_;
     # $date should be in yyyymmdd format
-    my $fs = $target->repos->fs;
+    my $fs = $target->{repos}->fs;
     my $yrev = $fs->youngest_rev;
 
-    my $rev = 0;
+    my ($rev,$last);
     traverse_history (
         root        => $fs->revision_root($yrev),
         path        => $target->path,
@@ -1099,28 +979,29 @@ sub find_date_rev {
             my $revdate = $props->{'svn:date'};
             $revdate =~ s/T.*$//; $revdate =~ s/-//g;
             if($date > $revdate) {
-                $rev = $_[1];
+                $rev = ($last || $_[1]);
                 return 0;
             }
+            $last = $_[1];
             return 1;
         },
     );
-    return $rev;
+    return $rev || $last;
 }
 
 
 sub find_base_rev {
     my ($self,$target) = @_;
     die(loc("BASE can only be issued with a check-out path\n"))
-        unless $target->isa('SVK::Path::Checkout');
+        unless(defined($target->{copath}));
     my $rev = $self->{xd}{checkout}->get($target->copath)->{revision};
     return $rev;
 }
 
 sub find_head_rev {
     my ($self,$target) = @_;
-    $target = $target->as_depotpath;
-    my $fs = $target->repos->fs;
+    $target->as_depotpath;
+    my $fs = $target->{repos}->fs;
     my $yrev = $fs->youngest_rev;
     my $rev;
     traverse_history (
