@@ -8,7 +8,6 @@ use SVK::Util qw( abs2rel );
 
 sub options {
     ($_[0]->SUPER::options,
-     'force'	=> 'force',
      'K|keep-local'	=> 'keep');
 }
 
@@ -16,20 +15,11 @@ sub parse_arg {
     my ($self, @arg) = @_;
     return if $#arg < 0;
     my $target;
-    @arg = map { $self->{xd}->target_from_copath_maybe($_) } @arg;
-
-    # XXX: better check for @target being the same type
-    if (grep {$_->isa('SVK::Path::Checkout')} @arg) {
-	die loc("Mixed depotpath and checkoutpath not supported.\n")
-	    if grep {!$_->isa('SVK::Path::Checkout')} @arg;
-
-	return $self->{xd}->target_condensed(@arg);
+    if ($#arg == 0) {
+	$target = $self->arg_co_maybe ($arg[0]);
+	return $target unless $target->{copath};
     }
-
-    die loc("Delete for more than one depotpath is not supported yet.\n")
-	if scalar @arg > 1;
-
-    return @arg;
+    return $self->arg_condensed (@arg);
 }
 
 sub lock {
@@ -40,8 +30,8 @@ sub lock {
 sub do_delete_direct {
     my ($self, $target) = @_;
     my $m = $self->under_mirror ($target);
-    if ($m && $m->path eq $target->path) {
-	$m->detach;
+    if ($m && $m->{target_path} eq $target->path) {
+	$m->delete;
 	$target->refresh_revision;
 	undef $m;
     }
@@ -49,7 +39,7 @@ sub do_delete_direct {
     $self->get_commit_message ();
     $target->normalize;
     my ($anchor, $editor) = $self->get_dynamic_editor ($target);
-    my $rev = $target->revision;
+    my $rev = $target->{revision};
     $rev = $m->find_remote_rev ($rev) if $m;
     $editor->delete_entry (abs2rel ($target->path, $anchor => undef, '/'), $rev, 0);
     $self->adjust_anchor ($editor);
@@ -58,13 +48,11 @@ sub do_delete_direct {
 
 sub _ensure_mirror {
     my ($self, $target) = @_;
-
     my @m = $target->contains_mirror or return;
-
-    return if $#m == 0 && $m[0] eq $target->path_anchor;
+    return if !$target->{copath} && $#m == 0 && $m[0] eq $target->path;
 
     my $depotname = $target->depotname;
-    die loc("%1 contains mirror, remove explicitly: ", "/$depotname".$target->path_anchor).
+    die loc("%1 contains mirror, remove explicitly: ", "/$depotname".$target->path).
 	join(',', map { "/$depotname$_" } @m)."\n"
 }
 
@@ -73,9 +61,8 @@ sub run {
 
     $self->_ensure_mirror($target);
 
-    if ($target->isa('SVK::Path::Checkout')) {
-	$self->{xd}->do_delete( $target, no_rm => $self->{keep}, 
-		'force_delete' => $self->{force} );
+    if ($target->{copath}) {
+	$self->{xd}->do_delete ( %$target, no_rm => $self->{keep} );
     }
     else {
 	$self->do_delete_direct ( $target );
@@ -99,16 +86,9 @@ SVK::Command::Delete - Remove versioned item
 
 =head1 OPTIONS
 
+ -m [--message] MESSAGE	: specify commit message MESSAGE
+ -F [--file] FILENAME	: read commit message from FILENAME
  -K [--keep-local]      : do not remove the local file
- -m [--message] MESSAGE : specify commit message MESSAGE
- -F [--file] FILENAME   : read commit message from FILENAME
- --template             : use the specified message as the template to edit
- --encoding ENC         : treat -m/-F value as being in charset encoding ENC
- -P [--patch] NAME      : instead of commit, save this change as a patch
- -S [--sign]            : sign this change
- -C [--check-only]      : try operation but make no changes
- --direct               : commit directly even if the path is mirrored
- --force                : delete the file/directory even if modified
 
 =head1 AUTHORS
 
