@@ -8,6 +8,7 @@ use SVK::Patch;
 use SVK::Merge;
 use SVK::Editor::Merge;
 use SVK::I18N;
+use SVK::Util qw (resolve_svm_source);
 use SVK::Command::Log;
 
 
@@ -26,18 +27,7 @@ sub options {
 }
 
 sub parse_arg {
-    my ($self, @args) = @_;
-    my $cmd = shift @args or return;
-
-    # Try to find a subcommand for invocations such as "svk patch ls"
-    my %options = $self->options;
-    while (my ($names, $canonical) = each %options) {
-        $cmd =~ /\A$names\z/ or next;
-        bless($self, "SVK::Command::Patch::$canonical");
-        return($self->parse_arg(@args));
-    }
-
-    # Can't find a subcommand - return help
+    # always return help
     return;
 }
 
@@ -87,7 +77,7 @@ use base qw/SVK::Command::Patch::FileRequired/;
 
 sub run {
     my ($self, $patch) = @_;
-    print YAML::Syck::Dump ($patch);
+    print YAML::Dump ($patch);
     return;
 }
 
@@ -159,6 +149,7 @@ use base qw/SVK::Command::Patch/;
 sub parse_arg { undef }
 sub run {
     my ($self) = @_;
+    my (undef, undef, $repos) = $self->{xd}->find_repos ('//', 1);
     opendir my $dir, $self->{xd}->patch_directory;
     foreach my $file (readdir ($dir)) {
 	next if $file =~ /^\./;
@@ -183,11 +174,10 @@ sub run {
     my $mergecmd = $self->command ('merge');
     $mergecmd->getopt (\@args);
     my $dst = $self->arg_co_maybe ($args[0] || '');
-    $self->lock_target ($dst) if $dst->isa('SVK::Path::Checkout');
+    $self->lock_target ($dst) if $dst->{copath};
     my $ticket;
-    $mergecmd->get_commit_message ($patch->{log})
-	unless $dst->isa('SVK::Path::Checkout');
-    my $merge = SVK::Merge->new (%$mergecmd, dst => $dst, repos => $dst->repos);
+    $mergecmd->get_commit_message ($patch->{log}) unless $dst->{copath};
+    my $merge = SVK::Merge->new (%$mergecmd, dst => $dst, repos => $dst->{repos});
     $ticket = sub { $merge->get_new_ticket (SVK::Merge::Info->new ($patch->{ticket})) }
 	if $patch->{ticket} && $dst->universal->same_resource ($patch->{target});
     $patch->apply_to ($dst, $mergecmd->get_editor ($dst),
@@ -216,7 +206,7 @@ SVK::Command::Patch - Manage patches
 
 =head1 OPTIONS
 
- --depot DEPOTNAME      : operate on a depot other than the default one
+ None
 
 =head1 DESCRIPTION
 
@@ -292,13 +282,9 @@ To see the current version of a specific patch, run:
     svk patch --view Foo
 
 When you're done with a patch and don't want it hanging around anymore,
-run:
+run
     svk patch --delete Foo
 
-To apply a patch to the repository that someone else has sent you, run:
-
-    svk patch --apply - < contributed_feature.patch
-
 =head1 AUTHORS
 
 Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
