@@ -23,6 +23,7 @@ sub parse_arg {
 
 sub run {
     my ($self, @arg) = @_;
+
     die loc ("Revision required.\n")
 	if $self->{revprop} && !defined $self->{rev};
 
@@ -30,14 +31,14 @@ sub run {
         if ($self->{revprop}) {
             $self->_show_props
 		( $target,
-		  $target->repos->fs->revision_proplist($self->{rev}),
+		  $target->{repos}->fs->revision_proplist($self->{rev}),
 		  $self->{rev}
 		);
             next;
         }
 
-	$target = $target->as_depotpath ($self->{rev}) if defined $self->{rev};
-        $self->_show_props( $target, $target->root->node_proplist($target->path) );
+	$target->as_depotpath ($self->{rev}) if defined $self->{rev};
+        $self->_show_props ($target, $self->{xd}->do_proplist ( $target ));
     }
 
     return;
@@ -52,7 +53,7 @@ sub _show_props {
         print loc("Unversioned properties on revision %1:\n", $rev);
     }
     else {
-        print loc("Properties on %1:\n", length $target->report ? $target->report : '.');
+        print loc("Properties on %1:\n", $target->{report} || '.');
     }
 
     for my $key (sort keys %$props) {
@@ -69,13 +70,13 @@ sub _arg_revprop {
 sub _proplist {
     my ($self, $target) = @_;
 
-    return $target->repos->fs->revision_proplist($self->{rev})
+    return $target->{repos}->fs->revision_proplist($self->{rev})
 	if $self->{revprop};
 
     if (defined $self->{rev}) {
-        $target = $target->as_depotpath ($self->{rev});
+        $target->as_depotpath ($self->{rev});
     }
-    return $target->root->node_proplist($target->path);
+    return $self->{xd}->do_proplist ($target);
 }
 
 
@@ -95,8 +96,9 @@ SVK::Command::Proplist - List all properties on files or dirs
 
  -R [--recursive]       : descend recursively
  -v [--verbose]         : print extra information
- -r [--revision] REV    : act on revision REV instead of the head revision
+ -r [--revision] REV	: act on revision REV instead of the head revision
  --revprop              : operate on a revision property (use with -r)
+ --direct               : commit directly even if the path is mirrored
 
 =head1 AUTHORS
 
