@@ -6,6 +6,7 @@ use base qw( SVK::Command );
 use constant opt_recursive => 1;
 use SVK::XD;
 use SVK::I18N;
+use SVK::Util qw( HAS_SVN_MIRROR );
 
 sub options {
     ('r|revision=s'    => 'rev',
@@ -31,29 +32,19 @@ sub lock {
 sub run {
     my ($self, @arg) = @_;
 
-    die loc ("--check-only cannot be used in conjunction with --merge.\n")
-        if defined $self->{check_only} && $self->{merge};
+    die loc ("--check-only cannot be used in conjunction with --sync or --merge.\n")
+        if defined $self->{check_only} && ($self->{merge} || $self->{sync});
 
     die loc ("--revision cannot be used in conjunction with --sync or --merge.\n")
 	if defined $self->{rev} && ($self->{merge} || $self->{sync});
 
     for my $target (@arg) {
-	my $update_target = $target->source->new;
-	$update_target->path($self->{update_target_path})
-	    if defined $self->{update_target_path};
-
-	my $rev = defined $self->{rev} ?
-	    $self->resolve_revision($target->new,$self->{rev}) :
-	    $target->repos->fs->youngest_rev;
-
-	if ($update_target->isa('SVK::Path::View')) {
-	    # always use the latest view layout
-	    $update_target->refresh_revision;
-	    $update_target->source->revision($rev);
-	}
-        else {
-	    $update_target->revision($rev);
-	}
+	my $update_target = SVK::Target->new
+	    ( %$target,
+	      path => $self->{update_target_path} || $target->{path},
+	      revision => defined $self->{rev} ? $self->resolve_revision($target->clone,$self->{rev}) : $target->{repos}->fs->youngest_rev,
+	      copath => undef
+	    );
 
         # Because merging under the copy anchor is unsafe, we always merge
         # to the most immediate copy anchor under copath root.
@@ -65,10 +56,15 @@ sub run {
         delete $self->{merge} if !$copied_from;
 
         if ($self->{sync}) {
+            die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
+
             # Because syncing under the mirror anchor is impossible,
             # we always sync from the mirror anchor.
-            my $m = $sync_target->is_mirrored;
-            $m->run if $m;
+            my ($m, $mpath) = SVN::Mirror::is_mirrored (
+                $sync_target->{repos},
+                $sync_target->{path}
+            );
+            $m->run if $m->{source};
         }
 
         if ($self->{merge}) {
@@ -90,74 +86,51 @@ sub run {
 
 sub do_update {
     my ($self, $cotarget, $update_target) = @_;
-    my $pool = SVN::Pool->new_default;
-    my $xdroot = $cotarget->create_xd_root;
-    my $newroot = $update_target->root;
+    my ($xdroot, $newroot) = map { $_->root ($self->{xd}) } ($cotarget, $update_target);
     # unanchorified
-    my $report = $cotarget->report;
-    my $kind = $newroot->check_path ($update_target->path);
-    if ($kind == $SVN::Node::none) {
-	# if update target doesn't exist, only allows updating from
-	# something that exist.
-	die loc("Path %1 does not exist.\n", $update_target->depotpath)
-	    unless $xdroot->check_path($cotarget->path);
-
-	$cotarget->anchorify;
-	$update_target->anchorify;
-	# still in the checkout
-	if ($self->{xd}{checkout}->get($cotarget->copath)->{depotpath}) {
-	    $kind = $newroot->check_path($update_target->path_anchor);
-	}
-    }
-
-    my $content_revision = $update_target->isa('SVK::Path::View') ?
-	$update_target->source->revision : $update_target->revision;
-    print loc("Syncing %1(%2) in %3 to %4.\n", $cotarget->depotpath,
-	      $cotarget->path_anchor, $cotarget->copath,
-	      $content_revision);
+    my ($path, $copath) = @{$cotarget}{qw/path copath/};
+    my $report = $cotarget->{report};
+    my $kind = $newroot->check_path ($update_target->{path});
+    die loc("path %1 does not exist.\n", $update_target->{path})
+	if $kind == $SVN::Node::none;
 
+    print loc("Syncing %1(%2) in %3 to %4.\n", @{$cotarget}{qw( depotpath path copath )},
+	      $update_target->{revision});
     if ($kind == $SVN::Node::file ) {
 	$cotarget->anchorify;
 	$update_target->anchorify;
 	# can't use $cotarget->{path} directly since the (rev0, /) hack
-	$cotarget->source->{targets}[0] = $cotarget->{copath_target};
+	($path, $copath) = @{$cotarget}{qw/path copath/};
+	$cotarget->{targets}[0] = $cotarget->{copath_target};
     }
-    my $base = $cotarget->as_depotpath;
-    $base = $base->new(path => '/')
+    my $base = $cotarget;
+    $base = $base->new (path => '/')
 	if $xdroot->check_path ($base->path) == $SVN::Node::none;
-
-    unless (-e $cotarget->copath) {
+    unless (-e $cotarget->{copath}) {
 	die loc ("Checkout directory gone. Use 'checkout %1 %2' instead.\n",
-		 $update_target->depotpath, $cotarget->report)
-	    unless $base->path_anchor eq '/';
-	mkdir ($cotarget->copath) or
-	    die loc ("Can't create directory %1 for checkout: %2.\n", $cotarget->report, $!);
+		 $update_target->{depotpath}, $cotarget->{report})
+	    unless $base->{path} eq '/';
+	mkdir ($cotarget->{copath}) or
+	    die loc ("Can't create directory %1 for checkout: %2.\n", $cotarget->{report}, $!);
     }
 
     my $notify = SVK::Notify->new_with_report
-	($report, $cotarget->path_target, 1);
+	($report, $cotarget->{targets}[0], 1);
     $notify->{quiet}++ if $self->{quiet};
     my $merge = SVK::Merge->new
-	(repos => $cotarget->repos, base => $base, base_root => $xdroot,
+	(repos => $cotarget->{repos}, base => $base, base_root => $xdroot,
 	 no_recurse => !$self->{recursive}, notify => $notify, nodelay => 1,
 	 src => $update_target, dst => $cotarget, check_only => $self->{check_only},
-	 auto => 1, # not to print track-rename hint
 	 xd => $self->{xd});
-    my ($editor, $inspector, %cb) = $cotarget->get_editor
-	( ignore_checksum => 1,
-	  check_only => $self->{check_only},
-	  store_path => $update_target->path_anchor,
-	  update => $self->{check_only} ? 0 : 1,
-	  newroot => $newroot,
-	  revision => $content_revision,
-	);
-    $merge->run($editor, %cb, inspector => $inspector);
-
-    if ($update_target->isa('SVK::Path::View')) {
-	$self->{xd}{checkout}->store
-	    ($cotarget->copath,
-	     {depotpath => $update_target->depotpath});
-    }
+    $merge->run ($self->{xd}->get_editor (copath => $copath, path => $path,
+					  store_path => $update_target->path,
+					  check_only => $self->{check_only},
+					  ignore_checksum => 1,
+					  oldroot => $xdroot, newroot => $newroot,
+					  revision => $update_target->{revision},
+					  anchor => $cotarget->{path},
+					  target => $cotarget->{targets}[0] || '',
+					  update => $self->{check_only} ? 0 : 1));
 }
 
 1;
@@ -174,12 +147,12 @@ SVK::Command::Update - Bring changes from repository to checkout copies
 
 =head1 OPTIONS
 
- -r [--revision] REV    : act on revision REV instead of the head revision
+ -r [--revision] REV	: act on revision REV instead of the head revision
  -N [--non-recursive]   : do not descend recursively
  -C [--check-only]      : try operation but make no changes
  -s [--sync]            : synchronize mirrored sources before update
  -m [--merge]           : smerge from copied sources before update
- -q [--quiet]           : print as little as possible
+ -q [--quiet]           : quiet mode
 
 =head1 DESCRIPTION
 
