@@ -9,21 +9,16 @@ our @EXPORT_OK = qw(
 
     get_encoding get_encoder from_native to_native
 
-    find_svm_source traverse_history
+    find_local_mirror find_svm_source resolve_svm_source traverse_history
     find_prev_copy
 
     read_file write_file slurp_fh md5_fh bsd_glob mimetype mimetype_is_text
-    is_binary_file
 
     abs_path abs2rel catdir catfile catpath devnull dirname get_anchor 
     move_path make_path splitpath splitdir tmpdir tmpfile get_depot_anchor
     catdepot abs_path_noexist 
 
     is_symlink is_executable is_uri can_run
-
-    str2time time2str reformat_svn_date
-
-    find_dotsvk
 );
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 
@@ -31,7 +26,8 @@ use SVK::Version;  our $VERSION = $SVK::VERSION;
 use Config ();
 use SVK::I18N;
 use SVN::Core;
-use autouse 'Encode'            => qw(resolve_alias($) decode encode);
+use SVN::Ra;
+use autouse 'Encode'            => qw(resolve_alias decode encode);
 use File::Glob qw(bsd_glob);
 use autouse 'File::Basename' 	=> qw(dirname);
 use autouse 'File::Spec::Functions' => 
@@ -327,13 +323,72 @@ sub to_native {
     return;
 }
 
-sub find_svm_source { # DEPRECATED: use SVK::Path->universal, only used in SVK::Command now.
+=head2 Mirror Handling
+
+=head3 find_local_mirror ($repos, $uuid, $path, $rev)
+
+XXX Undocumented
+
+=cut
+
+sub find_local_mirror {
+    my ($repos, $uuid, $path, $rev) = @_;
+    my $myuuid = $repos->fs->get_uuid;
+    return unless HAS_SVN_MIRROR && $uuid ne $myuuid;
+    my ($m, $mpath) = SVN::Mirror::has_local ($repos, "$uuid:$path");
+    return ("$m->{target_path}$mpath",
+	    $rev ? $m->find_local_rev ($rev) : $rev) if $m;
+}
+
+=head3 find_svm_source ($repos, $path, $rev)
+
+XXX Undocumented
+
+=cut
+
+sub find_svm_source {
     my ($repos, $path, $rev) = @_;
-    my $t = SVK::Path->real_new({ depot => SVK::Depot->new({repos => $repos}),
-                                  path => $path, revision => $rev });
-    $t->refresh_revision unless $rev;
-    my $u = $t->universal;
-    return map { $u->$_ } qw(uuid path rev);
+    my $fs = $repos->fs;
+    $rev ||= $fs->youngest_rev;
+    my $root = $fs->revision_root ($rev);
+    my ($uuid, $m, $mpath);
+
+    if (HAS_SVN_MIRROR) {
+	($m, $mpath) = SVN::Mirror::is_mirrored ($repos, $path);
+    }
+
+    if ($m) {
+	# XXX: we should normalize $rev before calling find_svm_source
+	$rev = ($root->node_history($path)->prev(0)->location)[1]
+	    unless $rev == $root->node_created_rev ($path);
+	$rev = $m->find_remote_rev ($rev);
+	$path =~ s/\Q$mpath\E$//;
+	$uuid = $m->{source_uuid};
+	$path = $m->{source_path}.$mpath;
+	$path ||= '/';
+    }
+    else {
+	$uuid = $fs->get_uuid;
+	$rev = ($root->node_history ($path)->prev (0)->location)[1];
+    }
+
+    return ($uuid, $path, $rev);
+}
+
+=head3 resolve_svm_source ($repos, $uuid, $path)
+
+XXX Undocumented
+
+=cut
+
+sub resolve_svm_source {
+    my ($repos, $uuid, $path) = @_;
+    my $myuuid = $repos->fs->get_uuid;
+    return ($path) if ($uuid eq $myuuid);
+    return unless HAS_SVN_MIRROR;
+    my ($m, $mpath) = SVN::Mirror::has_local ($repos, "$uuid:$path");
+    return unless $m;
+    return ("$m->{target_path}$mpath", $m);
 }
 
 =head2 File Content Manipulation
@@ -402,8 +457,8 @@ Calculate MD5 checksum for data in the input filehandle.
 }
 
 sub md5_fh {
-    require Digest::MD5;
     my $fh = shift;
+
     my $ctx = Digest::MD5->new;
     $ctx->addfile($fh);
 
@@ -418,20 +473,22 @@ is missing on the system.
 =cut
 
 sub mimetype {
-    my ($filename) = @_;
-    my $mm if 0;  # C<state $mm>, yuck
+    my $fh = shift;
 
-    # find an implementation module if necessary
-    if ( !$mm ) {
-        my $module = $ENV{SVKMIME} || 'Internal';
-        $module =~ s/:://;
-        $module = "SVK::MimeDetect::$module";
-        eval "require $module";
-        die $@ if $@;
-        $mm = $module->new();
+    return 'text/plain' if -z $fh;
+
+    binmode($fh);
+    read $fh, my $data, 16*1024 or return undef;
+
+    my $type = File::Type->checktype_contents($data);
+
+    # On fallback, use the same logic as File::MimeInfo to detect text
+    if ($type eq 'application/octet-stream') {
+        substr($data, 0, 32) =~ m/[\x00-\x07\x0B\x0E-\x1A\x1C-\x1F]/
+            or return 'text/plain';
     }
 
-    return $mm->checktype_filename($filename);
+    return $type;
 }
 
 =head3 mimetype_is_text ($mimetype)
@@ -455,21 +512,6 @@ sub mimetype_is_text {
                          |image/x-x(?:bit|pix)map)$}x;
 }
 
-=head3 is_binary_file ($filename OR $filehandle)
-
-Returns true if the given file or filehandle contains binary data.  Otherwise,
-returns false.
-
-=cut
-
-sub is_binary_file {
-    my ($file) = @_;
-
-    # let Perl do the hard work
-    return 1 if -f $file && !-T _;  # !-T handles empty files correctly
-    return;
-}
-
 =head2 Path and Filename Handling
 
 =head3 abspath ($path)
@@ -535,11 +577,11 @@ sub abs_path_noexist {
 Replace the base directory in the native pathname to another base directory
 and return the result.
 
-If the pathname is not under C<$old_basedir>, it is returned unmodified.
+If the pathname is not under C<$old_basedir>, it is not unmodified.
 
 If C<$new_basedir> is an empty string, removes the old base directory but
-keeps the leading slash.  If C<$new_basedir> is C<undef>, also removes
-the leading slash.
+keeps the trailing slash.  If C<$new_basedir> is C<undef>, also removes
+the trailing slash.
 
 By default, the return value of this function will use C<$SEP> as its
 path separator.  Setting C<$sep> to C</> will turn native path separators
@@ -559,9 +601,6 @@ sub abs2rel {
         $rel = catdir($new_basedir, $rel);
     }
 
-    # resemble file::spec pre-3.13 behaviour, return empty string.
-    return '' if $rel eq '.';
-
     $rel =~ s/\Q$SEP/$sep/go if $sep and $SEP ne $sep;
     return $rel;
 }
@@ -654,12 +693,12 @@ sub make_path {
     return undef if !defined($path) or -d $path;
 
     require File::Path;
-    my @ret = eval { File::Path::mkpath([$path]) };
+    my $ret = eval { File::Path::mkpath([$path]) };
     if ($@) {
-	$@ =~ s/ at .*//;
+	$@ =~ s/ at .*/\n/;
 	die $@;
     }
-    return @ret;
+    return $ret;
 }
 
 =head3 splitpath ($path)
@@ -779,16 +818,6 @@ sub move_path {
     );
 }
 
-=head3 traverse_history (root => $fs_root, path => $path,
-    cross => $cross, callback => $cb($path, $revision))
-
-Traverse the history of $path in $fs_root backwards until the first
-copy, unless $cross is true.  We do cross renames regardless of the
-value of $cross.  We invoke $cb for each $path, $revision we
-encounter.  If cb returns a nonzero value we stop traversing as well.
-
-=cut
-
 sub traverse_history {
     my %args = @_;
 
@@ -796,60 +825,14 @@ sub traverse_history {
     my $new_pool = SVN::Pool->new;
     my $spool = SVN::Pool->new_default;
 
-    my ($root, $path) = @args{qw/root path/};
-    # If the root is txn root, get a similar one.
-    # XXX: We actually want to move this to SVK::Path::, and
-    # svk::checkout should respect copies on checkout
-    if ($root->can('txn') && $root->txn) {
-	($root, $path) = $root->get_revision_root
-	    ($path, $root->txn->base_revision );
-    }
-
-    my $hist = $root->node_history ($path, $old_pool);
+    my $hist = $args{root}->node_history ($args{path}, $old_pool);
     my $rv;
-    my $revision;
 
-    while (1) {
-        my $ohist = $hist;
-        $hist = $hist->prev(($args{cross} || 0), $new_pool);
-        if (!$hist) {
-            last if $args{cross};
-            last unless $hist = $ohist->prev((1), $new_pool);
-            # We are not supposed to cross copies, ($path,$revision)
-            # refers to a node in $ohist that is a copy and that has a
-            # prev if we ask svn to traverse copies.
-            # Let's find out if the copy was actually a rename instead
-            # of a copy.
-            my $root = $root->fs->revision_root($revision, $spool);
-            my $frompath;
-            my $fromrev = -1;
-            # We know that $path was a real copy and it that it has a
-            # prev, so find the node from which it was copied.
-            do {
-                ($fromrev, $frompath) = $root->copied_from($path, $spool);
-            } until ($fromrev >= 0 || !($path =~ s{/[^/]*$}{}));
-            die "Assertion failed: $path in $revision isn't a copy."
-                if $fromrev < 0;
-            # Ok, $path in $root was a copy of ($frompath,$fromrev).
-            # If $frompath was deleted in $root then the copy was really
-            # a rename.
-            my $entry = $root->paths_changed($spool)->{$frompath};
-            last unless $entry &&
-                $entry->change_kind == $SVN::Fs::PathChange::delete;
-
-            # XXX Do we need to worry about a parent of $frompath having
-            # been deleted instead?  If so the 2 lines below might work as
-            # an alternative, to the previous 3 lines.  However this also
-            # treats a delete followed by a copy of an older revision in
-            # two separate commits as a rename, which technically it's not.
-            #last unless $root->check_path($frompath, $spool) ==
-            #    $SVN::Node::none;
-        }
-        ($path, $revision) = $hist->location ($new_pool);
-        $old_pool->clear;
-        $rv = $args{callback}->($path, $revision);
+    while ($hist = $hist->prev(($args{cross} || 0), $new_pool)) {
+        $rv = $args{callback}->($hist->location ($new_pool));
         last if !$rv;
-        $spool->clear;
+        $old_pool->clear;
+	$spool->clear;
         ($old_pool, $new_pool) = ($new_pool, $old_pool);
     }
 
@@ -907,74 +890,6 @@ sub find_prev_copy {
     return ($root, $copy);
 }
 
-sub reformat_svn_date {
-    my ($format, $svn_date) = @_;
-    return time2str($format, str2time($svn_date));
-}
-
-sub str2time {
-    require Time::Local;
-    my ($year, $month, $day, $hh, $mm, $ss) = split /[-T:]/, $_[0];
-    $year -= 1900;
-    $month--;
-    chop($ss);  # remove the 'Z'
-    my $zone = 0;  # UTC
-
-    my @lt = localtime(time);
-
-    my $frac = $ss - int($ss);
-    $ss = int $ss;
-
-    for ( $year, $month, $day, $hh, $mm, $ss ) {
-        return undef unless defined($_) 
-    }
-    return undef
-      unless ( $month <= 11
-        && $day >= 1
-        && $day <= 31
-        && $hh <= 23
-        && $mm <= 59
-        && $ss <= 59 );
-
-    my $result;
-
-    $result = eval {
-        local $SIG{__DIE__} = sub { };    # Ick!
-        Time::Local::timegm( $ss, $mm, $hh, $day, $month, $year );
-    };
-    return undef
-        if !defined $result
-        or $result == -1
-        && join( "", $ss, $mm, $hh, $day, $month, $year ) ne "595923311169";
-
-    return $result + $frac;
-}
-
-sub time2str {
-    my ($format, $time) = @_;
-    require POSIX;
-    return POSIX::strftime($format, localtime($time) );
-}
-
-
-sub find_dotsvk {
-    require Cwd;
-    require Path::Class;
-
-    my $p = Path::Class::Dir->new( Cwd::cwd() );
-
-    my $prev = "not $p";
-    my $found = q{};
-    while ( $p && $p ne $prev && -r $p ) {
-	$prev = $p;
-	my $svk = $p->subdir('.svk');
-	return $svk if -e $svk;
-	$p = $p->parent();
-    }
-
-    return
-}
-
 1;
 
 __END__
